@@ -26,6 +26,8 @@
 
 #include "protocol.h"
 #include "protocol_util.h"
+#include "lib/util/util.h"
+#include "lib/util/smb_strtox.h"
 
 static struct {
 	enum ctdb_runstate runstate;
@@ -117,6 +119,7 @@ int ctdb_sock_addr_to_buf(char *buf, socklen_t buflen,
 			  ctdb_sock_addr *addr, bool with_port)
 {
 	const char *t;
+	size_t len = 0;
 
 	switch (addr->sa.sa_family) {
 	case AF_INET:
@@ -125,15 +128,36 @@ int ctdb_sock_addr_to_buf(char *buf, socklen_t buflen,
 		if (t == NULL) {
 			return errno;
 		}
+		if (with_port) {
+			len = strlen(buf);
+		}
 		break;
 
-	case AF_INET6:
-		t = inet_ntop(addr->ip6.sin6_family, &addr->ip6.sin6_addr,
-			      buf, buflen);
+	case AF_INET6: {
+		char tmp[INET6_ADDRSTRLEN];
+
+		t = inet_ntop(addr->ip6.sin6_family,
+			      &addr->ip6.sin6_addr,
+			      tmp,
+			      sizeof(tmp));
 		if (t == NULL) {
 			return errno;
 		}
+
+		if (with_port) {
+			int ret = snprintf(buf, buflen, "[%s]", tmp);
+			if (ret < 0) {
+				return ENOSPC;
+			}
+			len = (size_t)ret;
+		} else {
+			len = strlcpy(buf, tmp, buflen);
+		}
+		if (len >= buflen){
+			return ENOSPC;
+		}
 		break;
+	}
 
 	default:
 		return EAFNOSUPPORT;
@@ -141,12 +165,11 @@ int ctdb_sock_addr_to_buf(char *buf, socklen_t buflen,
 	}
 
 	if (with_port) {
-		size_t len = strlen(buf);
 		int ret;
 
 		ret = snprintf(buf+len, buflen-len,
 			       ":%u", ctdb_sock_addr_port(addr));
-		if (ret >= buflen-len) {
+		if (ret < 0 || (size_t)ret >= buflen-len) {
 			return ENOSPC;
 		}
 	}
@@ -154,8 +177,9 @@ int ctdb_sock_addr_to_buf(char *buf, socklen_t buflen,
 	return 0;
 }
 
-const char *ctdb_sock_addr_to_string(TALLOC_CTX *mem_ctx,
-				     ctdb_sock_addr *addr, bool with_port)
+char *ctdb_sock_addr_to_string(TALLOC_CTX *mem_ctx,
+			       ctdb_sock_addr *addr,
+			       bool with_port)
 {
 	size_t len = 64;
 	char *cip;
@@ -227,16 +251,29 @@ static int ip_from_string(const char *str, ctdb_sock_addr *addr)
 
 	/* IPv4 or IPv6 address?
 	 *
-	 * Use rindex() because we need the right-most ':' below for
+	 * Use strrchr() because we need the right-most ':' below for
 	 * IPv4-mapped IPv6 addresses anyway...
 	 */
-	p = rindex(str, ':');
+	p = strrchr(str, ':');
 	if (p == NULL) {
 		ret = ipv4_from_string(str, &addr->ip);
 	} else {
-		uint8_t ipv4_mapped_prefix[12] = {
+		static const uint8_t ipv4_mapped_prefix[12] = {
 			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0xff, 0xff
 		};
+		size_t len = strlen(str);
+		char s[64];
+
+		len = strlcpy(s, str, sizeof(s));
+		if (len >= sizeof(s)) {
+			return EINVAL;
+		}
+
+		if ((len >= 2) && (s[0] == '[') && (s[len-1] == ']')) {
+			s[len-1] = '\0';
+			str = s+1;
+			p = strrchr(str, ':');
+		}
 
 		ret = ipv6_from_string(str, &addr->ip6);
 		if (ret != 0) {
@@ -268,7 +305,6 @@ int ctdb_sock_addr_from_string(const char *str,
 	char *p;
 	char s[64]; /* Much longer than INET6_ADDRSTRLEN */
 	unsigned port;
-	char *endp = NULL;
 	size_t len;
 	int ret;
 
@@ -284,13 +320,13 @@ int ctdb_sock_addr_from_string(const char *str,
 		return EINVAL;
 	}
 
-	p = rindex(s, ':');
+	p = strrchr(s, ':');
 	if (p == NULL) {
 		return EINVAL;
 	}
 
-	port = strtoul(p+1, &endp, 10);
-	if (endp == p+1 || *endp != '\0') {
+	port = smb_strtoul(p+1, NULL, 10, &ret, SMB_STR_FULL_STR_CONV);
+	if (ret != 0) {
 		/* Empty string or trailing garbage */
 		return EINVAL;
 	}
@@ -310,9 +346,8 @@ int ctdb_sock_addr_mask_from_string(const char *str,
 	char *p;
 	char s[64]; /* Much longer than INET6_ADDRSTRLEN */
 	unsigned int m;
-	char *endp = NULL;
-	ssize_t len;
-	bool ret;
+	size_t len;
+	int ret = 0;
 
 	if (addr == NULL || mask == NULL) {
 		return EINVAL;
@@ -323,13 +358,13 @@ int ctdb_sock_addr_mask_from_string(const char *str,
 		return EINVAL;
 	}
 
-	p = rindex(s, '/');
+	p = strrchr(s, '/');
 	if (p == NULL) {
 		return EINVAL;
 	}
 
-	m = strtoul(p+1, &endp, 10);
-	if (endp == p+1 || *endp != '\0') {
+	m = smb_strtoul(p+1, NULL, 10, &ret, SMB_STR_FULL_STR_CONV);
+	if (ret != 0) {
 		/* Empty string or trailing garbage */
 		return EINVAL;
 	}
@@ -462,8 +497,11 @@ bool ctdb_sock_addr_same(const ctdb_sock_addr *addr1,
 	return (ctdb_sock_addr_cmp(addr1, addr2) == 0);
 }
 
-int ctdb_connection_to_buf(char *buf, size_t buflen,
-			   struct ctdb_connection *conn, bool client_first)
+int ctdb_connection_to_buf(char *buf,
+			   size_t buflen,
+			   struct ctdb_connection *conn,
+			   bool client_first,
+			   const char *sep)
 {
 	char server[64], client[64];
 	int ret;
@@ -481,20 +519,20 @@ int ctdb_connection_to_buf(char *buf, size_t buflen,
 	}
 
 	if (! client_first) {
-		ret = snprintf(buf, buflen, "%s %s", server, client);
+		ret = snprintf(buf, buflen, "%s%s%s", server, sep, client);
 	} else {
-		ret = snprintf(buf, buflen, "%s %s", client, server);
+		ret = snprintf(buf, buflen, "%s%s%s", client, sep, server);
 	}
-	if (ret >= buflen) {
+	if (ret < 0 || (size_t)ret >= buflen) {
 		return ENOSPC;
 	}
 
 	return 0;
 }
 
-const char *ctdb_connection_to_string(TALLOC_CTX *mem_ctx,
-				      struct ctdb_connection *conn,
-				      bool client_first)
+char *ctdb_connection_to_string(TALLOC_CTX *mem_ctx,
+				struct ctdb_connection *conn,
+				bool client_first)
 {
 	const size_t len = 128;
 	char *out;
@@ -505,7 +543,7 @@ const char *ctdb_connection_to_string(TALLOC_CTX *mem_ctx,
 		return NULL;
 	}
 
-	ret = ctdb_connection_to_buf(out, len, conn, client_first);
+	ret = ctdb_connection_to_buf(out, len, conn, client_first, " ");
 	if (ret != 0) {
 		talloc_free(out);
 		return NULL;
@@ -611,7 +649,7 @@ int ctdb_connection_list_sort(struct ctdb_connection_list *conn_list)
 	return 0;
 }
 
-const char *ctdb_connection_list_to_string(
+char *ctdb_connection_list_to_string(
 	TALLOC_CTX *mem_ctx,
 	struct ctdb_connection_list *conn_list, bool client_first)
 {
@@ -631,8 +669,11 @@ const char *ctdb_connection_list_to_string(
 		char buf[128];
 		int ret;
 
-		ret = ctdb_connection_to_buf(buf, sizeof(buf),
-					     &conn_list->conn[i], client_first);
+		ret = ctdb_connection_to_buf(buf,
+					     sizeof(buf),
+					     &conn_list->conn[i],
+					     client_first,
+					     " ");
 		if (ret != 0) {
 			talloc_free(out);
 			return NULL;
