@@ -33,6 +33,10 @@
 #include "librpc/gen_ndr/ndr_drsblobs.h"
 #include "auth/credentials/credentials.h"
 #include "libsmb/samlogon_cache.h"
+#include "lib/util/smb_strtox.h"
+#include "lib/util/string_wrappers.h"
+#include "lib/global_contexts.h"
+#include "librpc/gen_ndr/ndr_winbind_c.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_WINBIND
@@ -248,8 +252,6 @@ static NTSTATUS add_trusted_domain(const char *domain_name,
 	domain->last_seq_check = 0;
 	domain->initialized = false;
 	domain->online = is_internal_domain(sid);
-	domain->check_online_timeout = 0;
-	domain->dc_probe_pid = (pid_t)-1;
 	domain->domain_flags = trust_flags;
 	domain->domain_type = trust_type;
 	domain->domain_trust_attribs = trust_attribs;
@@ -374,7 +376,7 @@ bool domain_is_forest_root(const struct winbindd_domain *domain)
 
 struct trustdom_state {
 	struct winbindd_domain *domain;
-	struct winbindd_request request;
+	struct netr_DomainTrustList trusts;
 };
 
 static void trustdom_list_done(struct tevent_req *req);
@@ -383,8 +385,11 @@ static void rescan_forest_trusts( void );
 
 static void add_trusted_domains( struct winbindd_domain *domain )
 {
+	struct tevent_context *ev = global_event_context();
 	struct trustdom_state *state;
 	struct tevent_req *req;
+	const char *client_name = NULL;
+	pid_t client_pid;
 
 	state = talloc_zero(NULL, struct trustdom_state);
 	if (state == NULL) {
@@ -393,13 +398,18 @@ static void add_trusted_domains( struct winbindd_domain *domain )
 	}
 	state->domain = domain;
 
-	state->request.length = sizeof(state->request);
-	state->request.cmd = WINBINDD_LIST_TRUSTDOM;
+	/* Called from timer, not from a real client */
+	client_name = getprogname();
+	client_pid = getpid();
 
-	req = wb_domain_request_send(state, global_event_context(),
-				     domain, &state->request);
+	req = dcerpc_wbint_ListTrustedDomains_send(state,
+						   ev,
+						   dom_child_handle(domain),
+						   client_name,
+						   client_pid,
+						   &state->trusts);
 	if (req == NULL) {
-		DEBUG(1, ("wb_domain_request_send failed\n"));
+		DBG_ERR("dcerpc_wbint_ListTrustedDomains_send failed\n");
 		TALLOC_FREE(state);
 		return;
 	}
@@ -410,12 +420,9 @@ static void trustdom_list_done(struct tevent_req *req)
 {
 	struct trustdom_state *state = tevent_req_callback_data(
 		req, struct trustdom_state);
-	struct winbindd_response *response;
-	int res, err;
-	char *p;
-	ptrdiff_t extra_len;
 	bool within_forest = false;
-	NTSTATUS status;
+	NTSTATUS status, result;
+	uint32_t i;
 
 	/*
 	 * Only when we enumerate our primary domain
@@ -430,125 +437,25 @@ static void trustdom_list_done(struct tevent_req *req)
 		within_forest = true;
 	}
 
-	res = wb_domain_request_recv(req, state, &response, &err);
-	if ((res == -1) || (response->result != WINBINDD_OK)) {
-		DBG_WARNING("Could not receive trusts for domain %s\n",
-			    state->domain->name);
+	status = dcerpc_wbint_ListTrustedDomains_recv(req, state, &result);
+	if (any_nt_status_not_ok(status, result, &status)) {
+		DBG_WARNING("Could not receive trusts for domain %s: %s-%s\n",
+			    state->domain->name, nt_errstr(status),
+			    nt_errstr(result));
 		TALLOC_FREE(state);
 		return;
 	}
 
-	if (response->length < sizeof(struct winbindd_response)) {
-		DBG_ERR("ill-formed trustdom response - short length\n");
-		TALLOC_FREE(state);
-		return;
-	}
-
-	extra_len = response->length - sizeof(struct winbindd_response);
-
-	p = (char *)response->extra_data.data;
-
-	while ((p - (char *)response->extra_data.data) < extra_len) {
+	for (i=0; i<state->trusts.count; i++) {
+		struct netr_DomainTrust *trust = &state->trusts.array[i];
 		struct winbindd_domain *domain = NULL;
-		char *name, *q, *sidstr, *alt_name;
-		struct dom_sid sid;
-		uint32_t trust_type;
-		uint32_t trust_attribs;
-		uint32_t trust_flags;
-		int error = 0;
-
-		DBG_DEBUG("parsing response line '%s'\n", p);
-
-		name = p;
-
-		alt_name = strchr(p, '\\');
-		if (alt_name == NULL) {
-			DBG_ERR("Got invalid trustdom response\n");
-			break;
-		}
-
-		*alt_name = '\0';
-		alt_name += 1;
-
-		sidstr = strchr(alt_name, '\\');
-		if (sidstr == NULL) {
-			DBG_ERR("Got invalid trustdom response\n");
-			break;
-		}
-
-		*sidstr = '\0';
-		sidstr += 1;
-
-		/* use the real alt_name if we have one, else pass in NULL */
-		if (strequal(alt_name, "(null)")) {
-			alt_name = NULL;
-		}
-
-		q = strtok(sidstr, "\\");
-		if (q == NULL) {
-			DBG_ERR("Got invalid trustdom response\n");
-			break;
-		}
-
-		if (!string_to_sid(&sid, sidstr)) {
-			DEBUG(0, ("Got invalid trustdom response\n"));
-			break;
-		}
-
-		q = strtok(NULL, "\\");
-		if (q == NULL) {
-			DBG_ERR("Got invalid trustdom response\n");
-			break;
-		}
-
-		trust_flags = (uint32_t)smb_strtoul(q,
-						    NULL,
-						    10,
-						    &error,
-						    SMB_STR_STANDARD);
-		if (error != 0) {
-			DBG_ERR("Failed to convert trust_flags\n");
-			break;
-		}
-
-		q = strtok(NULL, "\\");
-		if (q == NULL) {
-			DBG_ERR("Got invalid trustdom response\n");
-			break;
-		}
-
-		trust_type = (uint32_t)smb_strtoul(q,
-						   NULL,
-						   10,
-						   &error,
-						   SMB_STR_STANDARD);
-		if (error != 0) {
-			DBG_ERR("Failed to convert trust_type\n");
-			break;
-		}
-
-		q = strtok(NULL, "\n");
-		if (q == NULL) {
-			DBG_ERR("Got invalid trustdom response\n");
-			break;
-		}
-
-		trust_attribs = (uint32_t)smb_strtoul(q,
-						      NULL,
-						      10,
-						      &error,
-						      SMB_STR_STANDARD);
-		if (error != 0) {
-			DBG_ERR("Failed to convert trust_attribs\n");
-			break;
-		}
 
 		if (!within_forest) {
-			trust_flags &= ~NETR_TRUST_FLAG_IN_FOREST;
+			trust->trust_flags &= ~NETR_TRUST_FLAG_IN_FOREST;
 		}
 
 		if (!state->domain->primary) {
-			trust_flags &= ~NETR_TRUST_FLAG_PRIMARY;
+			trust->trust_flags &= ~NETR_TRUST_FLAG_PRIMARY;
 		}
 
 		/*
@@ -557,12 +464,12 @@ static void trustdom_list_done(struct tevent_req *req)
 		 * This is important because we need the SID for sibling
 		 * domains.
 		 */
-		status = add_trusted_domain(name,
-					    alt_name,
-					    &sid,
-					    trust_type,
-					    trust_flags,
-					    trust_attribs,
+		status = add_trusted_domain(trust->netbios_name,
+					    trust->dns_name,
+					    trust->sid,
+					    trust->trust_type,
+					    trust->trust_flags,
+					    trust->trust_attributes,
 					    SEC_CHAN_NULL,
 					    find_default_route_domain(),
 					    &domain);
@@ -573,8 +480,6 @@ static void trustdom_list_done(struct tevent_req *req)
 				   nt_errstr(status));
 			return;
 		}
-
-		p = q + strlen(q) + 1;
 	}
 
 	/*
@@ -795,48 +700,91 @@ void rescan_trusted_domains(struct tevent_context *ev, struct tevent_timer *te,
 	return;
 }
 
-enum winbindd_result winbindd_dual_init_connection(struct winbindd_domain *domain,
-						   struct winbindd_cli_state *state)
+static void wbd_ping_dc_done(struct tevent_req *subreq);
+
+void winbindd_ping_offline_domains(struct tevent_context *ev,
+				   struct tevent_timer *te,
+				   struct timeval now,
+				   void *private_data)
 {
-	/* Ensure null termination */
-	state->request->domain_name
-		[sizeof(state->request->domain_name)-1]='\0';
-	state->request->data.init_conn.dcname
-		[sizeof(state->request->data.init_conn.dcname)-1]='\0';
+	struct winbindd_domain *domain = NULL;
 
-	if (strlen(state->request->data.init_conn.dcname) > 0) {
-		TALLOC_FREE(domain->dcname);
-		domain->dcname = talloc_strdup(domain,
-				state->request->data.init_conn.dcname);
-		if (domain->dcname == NULL) {
-			return WINBINDD_ERROR;
+	TALLOC_FREE(te);
+
+	for (domain = domain_list(); domain != NULL; domain = domain->next) {
+		DBG_DEBUG("Domain %s is %s\n",
+			  domain->name,
+			  domain->online ? "online" : "offline");
+
+		if (get_global_winbindd_state_offline()) {
+			DBG_DEBUG("We are globally offline, do nothing.\n");
+			break;
 		}
+
+		if (domain->online ||
+		    domain->check_online_event != NULL ||
+		    domain->secure_channel_type == SEC_CHAN_NULL) {
+			continue;
+		}
+
+		winbindd_flush_negative_conn_cache(domain);
+
+		domain->check_online_event =
+			dcerpc_wbint_PingDc_send(domain,
+						 ev,
+						 dom_child_handle(domain),
+						 &domain->ping_dcname);
+		if (domain->check_online_event == NULL) {
+			DBG_WARNING("Failed to schedule ping, no-memory\n");
+			continue;
+		}
+
+		tevent_req_set_callback(domain->check_online_event,
+					wbd_ping_dc_done, domain);
 	}
 
-	init_dc_connection(domain, false);
-
-	if (!domain->initialized) {
-		/* If we return error here we can't do any cached authentication,
-		   but we may be in disconnected mode and can't initialize correctly.
-		   Do what the previous code did and just return without initialization,
-		   once we go online we'll re-initialize.
-		*/
-		DEBUG(5, ("winbindd_dual_init_connection: %s returning without initialization "
-			"online = %d\n", domain->name, (int)domain->online ));
+	te = tevent_add_timer(ev,
+			      NULL,
+			      timeval_current_ofs(lp_winbind_reconnect_delay(),
+			                          0),
+			      winbindd_ping_offline_domains,
+			      NULL);
+	if (te == NULL) {
+		DBG_ERR("Failed to schedule winbindd_ping_offline_domains()\n");
 	}
 
-	fstrcpy(state->response->data.domain_info.name, domain->name);
-	fstrcpy(state->response->data.domain_info.alt_name, domain->alt_name);
-	sid_to_fstring(state->response->data.domain_info.sid, &domain->sid);
+	return;
+}
 
-	state->response->data.domain_info.native_mode
-		= domain->native_mode;
-	state->response->data.domain_info.active_directory
-		= domain->active_directory;
-	state->response->data.domain_info.primary
-		= domain->primary;
+static void wbd_ping_dc_done(struct tevent_req *subreq)
+{
+	struct winbindd_domain *domain =
+		tevent_req_callback_data(subreq,
+		struct winbindd_domain);
+	NTSTATUS status, result;
 
-	return WINBINDD_OK;
+	SMB_ASSERT(subreq == domain->check_online_event);
+	domain->check_online_event = NULL;
+
+	status = dcerpc_wbint_PingDc_recv(subreq, domain, &result);
+	TALLOC_FREE(subreq);
+	if (any_nt_status_not_ok(status, result, &status)) {
+		DBG_WARNING("dcerpc_wbint_PingDc_recv failed for domain: "
+			    "%s - %s\n",
+			    domain->name,
+			    nt_errstr(status));
+		return;
+	}
+
+	DBG_DEBUG("dcerpc_wbint_PingDc_recv() succeeded, "
+		  "domain: %s, dc-name: %s\n",
+                  domain->name,
+		  domain->ping_dcname);
+
+	talloc_free(discard_const(domain->ping_dcname));
+	domain->ping_dcname = NULL;
+
+	return;
 }
 
 static void wb_imsg_new_trusted_domain(struct imessaging_context *msg,
@@ -938,14 +886,6 @@ static bool add_trusted_domains_dc(void)
 					   nt_errstr(status));
 				return false;
 			}
-
-			/* Even in the parent winbindd we'll need to
-			   talk to the DC, so try and see if we can
-			   contact it. Theoretically this isn't necessary
-			   as the init_dc_connection() in init_child_recv()
-			   will do this, but we can start detecting the DC
-			   early here. */
-			set_domain_online_request(domain);
 		}
 
 		return true;
@@ -1011,16 +951,6 @@ static bool add_trusted_domains_dc(void)
 		}
 		domain->domain_type = domains[i]->trust_type;
 		domain->domain_trust_attribs = domains[i]->trust_attributes;
-
-		if (sec_chan_type != SEC_CHAN_NULL) {
-			/* Even in the parent winbindd we'll need to
-			   talk to the DC, so try and see if we can
-			   contact it. Theoretically this isn't necessary
-			   as the init_dc_connection() in init_child_recv()
-			   will do this, but we can start detecting the DC
-			   early here. */
-			set_domain_online_request(domain);
-		}
 	}
 
 	for (i = 0; i < num_domains; i++) {
@@ -1327,14 +1257,6 @@ bool init_domain_list(void)
 				"domain to winbindd's internal list\n");
 			return false;
 		}
-		/* Even in the parent winbindd we'll need to
-		   talk to the DC, so try and see if we can
-		   contact it. Theoretically this isn't necessary
-		   as the init_dc_connection() in init_child_recv()
-		   will do this, but we can start detecting the DC
-		   early here. */
-		set_domain_online_request(domain);
-
 	}
 
 	status = imessaging_register(winbind_imessaging_context(), NULL,
@@ -1562,7 +1484,7 @@ struct winbindd_domain *find_lookup_domain_from_name(const char *domain_name)
 	     strequal(domain_name, unix_groups_domain_name() ) )
 	{
 		/*
-		 * The "Unix User" and "Unix Group" domain our handled by
+		 * The "Unix User" and "Unix Group" domain are handled by
 		 * passdb
 		 */
 		return find_domain_from_name_noinit( get_global_sam_name() );
@@ -1721,6 +1643,9 @@ char *fill_domain_username_talloc(TALLOC_CTX *mem_ctx,
 	}
 
 	tmp_user = talloc_strdup(mem_ctx, user);
+	if (tmp_user == NULL) {
+		return NULL;
+	}
 	if (!strlower_m(tmp_user)) {
 		TALLOC_FREE(tmp_user);
 		return NULL;
@@ -1909,7 +1834,7 @@ NTSTATUS normalize_name_map(TALLOC_CTX *mem_ctx,
 ********************************************************************/
 
 NTSTATUS normalize_name_unmap(TALLOC_CTX *mem_ctx,
-			      char *name,
+			      const char *name,
 			      char **normalized)
 {
 	NTSTATUS nt_status;
@@ -2053,8 +1978,12 @@ static void winbindd_set_locator_kdc_env(const struct winbindd_domain *domain)
 		return;
 	}
 
-	if (asprintf_strupper_m(&var, "%s_%s", WINBINDD_LOCATOR_KDC_ADDRESS,
-				domain->alt_name) == -1) {
+	var = talloc_asprintf_strupper_m(
+		talloc_tos(),
+		"%s_%s",
+		WINBINDD_LOCATOR_KDC_ADDRESS,
+		domain->alt_name);
+	if (var == NULL) {
 		return;
 	}
 
@@ -2062,7 +1991,7 @@ static void winbindd_set_locator_kdc_env(const struct winbindd_domain *domain)
 		var, kdc));
 
 	setenv(var, kdc, 1);
-	free(var);
+	TALLOC_FREE(var);
 }
 
 /*********************************************************************
@@ -2090,13 +2019,17 @@ void winbindd_unset_locator_kdc_env(const struct winbindd_domain *domain)
 		return;
 	}
 
-	if (asprintf_strupper_m(&var, "%s_%s", WINBINDD_LOCATOR_KDC_ADDRESS,
-				domain->alt_name) == -1) {
+	var = talloc_asprintf_strupper_m(
+		talloc_tos(),
+		"%s_%s",
+		WINBINDD_LOCATOR_KDC_ADDRESS,
+		domain->alt_name);
+	if (var == NULL) {
 		return;
 	}
 
 	unsetenv(var);
-	free(var);
+	TALLOC_FREE(var);
 }
 #else
 
