@@ -25,7 +25,7 @@
 #include "smb_server/smb_server.h"
 #include "libcli/smb2/smb2.h"
 #include "smb_server/smb2/smb2_server.h"
-#include "smbd/service_stream.h"
+#include "samba/service_stream.h"
 #include "lib/stream/packet.h"
 #include "param/param.h"
 
@@ -374,22 +374,22 @@ static void reply_nt1(struct smbsrv_request *req, uint16_t choice)
 		DATA_BLOB blob = data_blob_null;
 		const char *oid;
 		NTSTATUS nt_status;
-		
-		server_credentials 
-			= cli_credentials_init(req);
-		if (!server_credentials) {
-			smbsrv_terminate_connection(req->smb_conn, "Failed to init server credentials\n");
-			return;
-		}
-		
-		cli_credentials_set_conf(server_credentials, req->smb_conn->lp_ctx);
-		nt_status = cli_credentials_set_machine_account(server_credentials, req->smb_conn->lp_ctx);
-		if (!NT_STATUS_IS_OK(nt_status)) {
-			DEBUG(10, ("Failed to obtain server credentials, perhaps a standalone server?: %s\n", nt_errstr(nt_status)));
+
+		server_credentials =
+			cli_credentials_init_server(req, req->smb_conn->lp_ctx);
+		if (server_credentials == NULL) {
+			DBG_DEBUG("Failed to obtain server credentials, "
+				  "perhaps a standalone server?\n");
 			/*
-			 * We keep the server_credentials as anonymous
-			 * this is required for the spoolss.notify test
+			 * Create anon server credentials for for the
+			 * spoolss.notify test.
 			 */
+			server_credentials = cli_credentials_init_anon(req);
+			if (server_credentials == NULL) {
+				smbsrv_terminate_connection(req->smb_conn,
+					"Failed to init server credentials\n");
+				return;
+			}
 		}
 
 		nt_status = samba_server_gensec_start(req,
