@@ -19,7 +19,6 @@
 """Blackbox tests for samba-tool drs."""
 
 import samba.tests
-import shutil
 import os
 import ldb
 import drs_base
@@ -42,15 +41,9 @@ class SambaToolDrsTests(drs_base.DrsBaseTestCase):
         self._enable_inbound_repl(self.dnsname_dc1)
         self._enable_inbound_repl(self.dnsname_dc2)
 
-        try:
-            shutil.rmtree(os.path.join(self.tempdir, "private"))
-            shutil.rmtree(os.path.join(self.tempdir, "etc"))
-            shutil.rmtree(os.path.join(self.tempdir, "msg.lock"))
-            os.remove(os.path.join(self.tempdir, "names.tdb"))
-            shutil.rmtree(os.path.join(self.tempdir, "state"))
-            shutil.rmtree(os.path.join(self.tempdir, "bind-dns"))
-        except Exception:
-            pass
+        self.rm_files('names.tdb', allow_missing=True)
+        self.rm_dirs('etc', 'msg.lock', 'private', 'state', 'bind-dns',
+                     allow_missing=True)
 
         super(SambaToolDrsTests, self).tearDown()
 
